import pickle
import time
import os

CACHE_FILE = "/tmp/lvl.pk"

while True:
    if os.path.exists(CACHE_FILE) and os.path.getsize(CACHE_FILE) > 0:
        try:
            with open(CACHE_FILE, 'rb') as file:
                cache = pickle.load(file)
                try:
                    if 'upass' in cache:
                        value, exp_time = cache['upass']
                        if exp_time < time.time():
                            with open(CACHE_FILE, 'wb') as clear_file:
                                pickle.dump({}, clear_file)
                except ValueError:
                    pass
        except (FileNotFoundError, EOFError, pickle.UnpicklingError):
            pass

    time.sleep(1)



import os
import platform
import yaml


def get_config_path():
    os_name = platform.system()
    config_paths = {
        'Linux': os.path.expanduser("~/.config/sqpass/"),
        'Windows': os.path.expanduser("")
    }
    return config_paths.get(os_name, "")


def create_default_config():
    passw_colors = {
        "passw_colors": {
            'index': '',
            'username': '',
            'appname': '',
            'password': '',

        }
    }
    return yaml.dump(passw_colors, indent=4)


def setup_config():
    CONFIG_PATH = get_config_path()
    if not os.path.exists(CONFIG_PATH):
        os.mkdir(CONFIG_PATH)
        path = os.path.join(CONFIG_PATH, "config.yaml")
        with open(path, "w") as f:
            f.write(create_default_config())
            print(f"Created config file at {path}")


def get_config():
    CONFIG_PATH = get_config_path()
    if os.path.exists(CONFIG_PATH):
        with open(os.path.join(CONFIG_PATH, "config.yaml"), "r") as f:
            data = yaml.load(f, Loader=yaml.SafeLoader)
            return data



from sqpass.create_db import clear_db
from sqpass.passwordManager.ciphers import encrypt, decrypt
from sqpass.passwordManager import tools
from sqpass.passwordManager.tools import opendb, dbfetch
from prettytable import PrettyTable
import sqlite3
import getpass
import sys
import os
import base64
import platform
import json


home_dir = os.path.expanduser("~")
db_path = os.path.join(home_dir, ".local", "share", "pass.db")

paths = {
    'Linux': os.path.expanduser("~/.local/share/pass.key"),
    'Windows': os.path.expanduser("~\\AppData\\pass.key")
}
pathname = paths.get(platform.system())


def readpass():
    if not os.path.exists(pathname):
        print('No Passkey found\nPerhaps you forgot to execute `sq-init`')
    else:
        with open(pathname, "r") as file:
            return file.read()


class DatabaseManager:
    """ Class Doc Goes Here"""

    def __init__(self, MasterPass, cryptedpass):
        self.User_Masterpass = readpass()
        check_pass = True if self.User_Masterpass == cryptedpass else False
        self.cryptedpass = cryptedpass if check_pass else None
        self.MasterPass = MasterPass if check_pass else None
        self.db = db_path
        if not check_pass:
            print("Not matched")
        try:
            self.termlines = os.get_terminal_size().columns
        except Exception:
            self.termlines = 100

    def update(self, table='passw', id=None):
        if table == "passw":
            lst = ["app_name", "username", "passwd"]
            lstinp = ["App Name: ", "Username: ", "Password: "]
        else:
            lst = ["title", "key"]
            lstinp = ["Title: ", "Key: "]
        userinp = []

        for idxlst, inp in enumerate(lst):
            if inp == "passwd":
                passInp = getpass.getpass("Password: ")
                if passInp:
                    salt, iv, enc = encrypt(passInp.encode(
                        'utf-8'), self.MasterPass.encode('utf-8'))
                    enc = json.dumps({'salt': salt, 'iv': iv, 'enc': enc})
            else:
                enc = input(lstinp[idxlst])
            userinp.append(enc)

        for idx, inp in enumerate(userinp):
            if inp != "":
                try:
                    with opendb(self.db) as cur:
                        update_query = f"""UPDATE '{table}'
                                            SET '{lst[idx]}' = '{inp}'
                                            WHERE id={id};"""
                        cur.execute(update_query)
                except sqlite3.Error as err:
                    print(err)
            else:
                pass

    def count(self, icase=False, table=None, column=None, cred=None):
        if column == "passwd":
            passlist = self._get_decrypted_passwords()
            self._print_passwords(passlist, cred)
        else:
            readquery = "SELECT COUNT(*) FROM {} WHERE {} = '{}';"
            if icase:
                readquery = "SELECT COUNT(*) FROM {} WHERE {} = '{}' COLLATE NOCASE;"
            with dbfetch(self.db, readquery.format(table, column, cred)) as row:
                if row:
                    print(row[0][0])

    def _get_decrypted_passwords(self):
        passlist = []
        with dbfetch(self.db, "SELECT username, app_name, passwd FROM passw;") as row:
            if row:
                for col in row:
                    data = json.loads(col[2])
                    salt, iv, ct = data.values()
                    decipher = decrypt(
                        salt, iv, ct, self.MasterPass.encode()).decode()
                    passlist.append(
                        {'Username': col[0], 'App': col[1], 'Pass': decipher})
        return passlist

    def _print_passwords(self, passlist, cred):
        headers = ['Username', 'App Name', 'Password']
        t = PrettyTable(headers)
        count = 0
        for passw in passlist:
            print()
            if cred == passw['Pass']:
                count += 1
                t.add_row([passw['Username'], passw['App'], passw['Pass']])
        col = ['' for _ in range(count-1)]
        t.add_column("Count", [str(count), *col])
        print(t)

    def filter(self, icase=False, username=None, appname=None, state="OR"):
        if username is None and appname is None:
            print("Please Define A Service Parameter")
        elif username is not None or appname is not None:
            if icase:
                readquery = """SELECT * FROM passw WHERE username = '{}' COLLATE NOCASE {} app_name = '{}' COLLATE NOCASE;"""
            else:
                readquery = """SELECT * FROM passw WHERE username = '{}' {} app_name = '{}';"""
            with dbfetch(self.db, readquery.format(username, state, appname)) as row:
                if row:
                    tools.print_box(row, self.MasterPass)

    def view_notes(self, sort="id", order="ASC", markdown=False, indexId=None):
        if indexId is None:
            readquery = f"SELECT * FROM notes ORDER BY {sort} {order};"
        else:
            readquery = f"SELECT * FROM notes WHERE id={indexId};"
        with dbfetch(self.db, readquery) as row:
            if row:
                if indexId is None:
                    notes = PrettyTable()
                    notes.field_names = ["Index", "Title", "Content"]
                    for idx, title, cont, dtime in row:
                        notes.add_row(
                            [idx, title, base64.b64decode(cont).decode()[:10]+"..."])
                    print(notes)

                else:
                    try:
                        title = row[0][1]
                        content = (
                            (base64.b64decode(row[0][2])).decode()).strip()
                        subtitle = row[0][3]

                        tools.print_note(content, title, subtitle)
                    except IndexError:
                        print("Not Available")

    def view_keys(self, sort="id", order="ASC", indexId=None, export=False):
        if indexId is None:
            readquery = f"SELECT * FROM keys ORDER BY {sort} {order};"
        else:
            readquery = f"SELECT * FROM keys WHERE id = {indexId};"
        with dbfetch(self.db, readquery) as row:
            if row and len(row) < 1:
                print("Not Available")
            else:
                keys = PrettyTable()
                keys.field_names = ["Index", "Title", "Key"]
                for r in row:
                    data = json.loads(r[2])
                    salt, iv, ct = data.values()
                    decipher = decrypt(
                        salt, iv, ct, self.MasterPass.encode()).decode()
                    keys.add_row([r[0], r[1], decipher])
                if export:
                    return keys
                print(keys)

    def view_userpasses(self, sort="id", order="ASC", indexId=None, countpass=False, export=False):
        if indexId is None:
            readquery = f"SELECT * FROM passw ORDER BY {sort} {order};"
        else:
            readquery = f"SELECT * FROM passw WHERE id = {indexId};"
        with dbfetch(self.db, readquery) as row:
            if row:
                content_table = tools.print_box(row, self.MasterPass)
                if export:
                    return content_table
                print(content_table)

    def insert(self, app_name=None, user_name=None):  # Pylint: disable=W0613
        insert_query = """INSERT INTO passw (app_name,username, passwd)
                            VALUES (:appname ,:u_name,:passw)"""
        app_name = input("App Name:")
        u_name = input("Username: ")
        salt, iv, enc = encrypt(getpass.getpass("Password: ").encode(
            'utf-8'), self.MasterPass.encode('utf-8'))
        data = json.dumps({'salt': salt, 'iv': iv, 'enc': enc})

        try:
            with opendb(self.db) as cur:
                cur.execute(insert_query, {
                    "appname": app_name,
                    "u_name": u_name,
                    "passw": data
                }
                )
            print(f"[+]Successfully Added for {u_name}")
        except sqlite3.Error as error:
            print("Failed to Insert Into Database", error)

    def keyins(self, title=None, silent=True):
        insert_query = """INSERT INTO keys (title, key)
                            VALUES (:key_title ,:key_pass)"""
        if title == 'None':
            title = input("Title: ")
        if silent:
            salt, iv, enc = encrypt(getpass.getpass(
                "Key: ").encode(), self.MasterPass.encode())
            data = json.dumps({'salt': salt, 'iv': iv, 'enc': enc})
        else:
            salt, iv, enc = encrypt(
                input("Key: ").encode(), self.MasterPass.encode())
            data = json.dumps({'salt': salt, 'iv': iv, 'enc': enc})
        try:
            with opendb(self.db) as cur:
                cur.execute(insert_query, {
                    "key_title": title,
                    "key_pass": data
                }
                )
            print()
            print(f"[+]Stored Key for {title}")
        except sqlite3.Error as error:
            print("Failed to Insert Into Database", error)

    def noteins(self, title=None):
        insert_query = """INSERT INTO notes (title,content, date)
                            VALUES (:note_title ,:note_content, datetime())"""
        if title == "None":
            title = input("Title:")
        centext = "BODY"
        save_text = " Save [CTRL + D]  "
        discard = " Cancel [CTRL + C] "

        eq = (self.termlines//2)-(len(save_text+centext+discard)//2)
        last_put = " "*(eq)+centext+" "*(eq)
        print("_"*self.termlines)
        print(discard, end="")
        print(last_put, end="")
        print(save_text, end="")
        print("-"*self.termlines)

        note_content = "".join(sys.stdin.readlines())
        contb64 = base64.b64encode(note_content.encode())
        try:
            with opendb(self.db) as cur:
                cur.execute(insert_query, {
                    "note_title": title,
                    "note_content": contb64,
                }
                )
            print()
            print(f"[+]Note Added")
        except sqlite3.Error as error:
            print("Failed to Insert Into Database", error)

    def remove_cd(self, table=None, u_id=None):

        query = f"DELETE FROM {table} WHERE id={u_id}"

        with opendb(self.db) as cur:
            cur.execute(query)

    def bigbang(self, boom=False, table_to_delete=None):
        if boom:
            clear_db()
        with opendb(self.db) as cur:
            cur.execute(f"DELETE FROM {table_to_delete}")

    def export(self, service=None, json=False, csv=False):
        backup = ""
        ext = ".json" if json else ".csv"
        if service == "passw":
            rawbac = self.view_userpasses(export=True)
            backup = rawbac.get_json_string() if json else rawbac.get_csv_string()

        elif service == "keys":
            rawbac = self.view_keys(export=True)
            backup = rawbac.get_json_string() if json else rawbac.get_csv_string()
        else:
            print("service name was not found")
        print(backup)
        with open("backup"+ext, "w") as f:
            for line in backup:
                if line != "\n":
                    f.write(line)



import base64
import hashlib
from Crypto.Cipher import AES
from Crypto.Random import get_random_bytes
from Crypto.Util.Padding import pad, unpad
from Crypto.Protocol.KDF import PBKDF2


def encrypt(data, master_key):
    salt = get_random_bytes(16)
    key = PBKDF2(master_key, salt, dkLen=32)
    cipher = AES.new(key, AES.MODE_CBC)
    ct_bytes = cipher.encrypt(pad(data, AES.block_size))

    iv = base64.b64encode(cipher.iv).decode('utf-8')
    ct = base64.b64encode(ct_bytes).decode('utf-8')
    salt = base64.b64encode(salt).decode('utf-8')

    return salt, iv, ct


def decrypt(salt, iv, ct, master_key):
    salt = base64.b64decode(salt)
    iv = base64.b64decode(iv)[:16]
    ct = base64.b64decode(ct)
    key = PBKDF2(master_key, salt, dkLen=32)
    cipher = AES.new(key, AES.MODE_CBC, iv=iv)
    decipher = unpad(cipher.decrypt(ct), AES.block_size)

    return decipher


def finalhash(msg, masterkey):
    masterkey = base64.b64encode(masterkey)
    IV = "agun".encode()*4
    cipher = AES.new(masterkey, AES.MODE_CFB, IV)
    encd = base64.b64encode(IV + cipher.encrypt(msg))
    return hashlib.sha256(encd).hexdigest()


def hashuser(masterkey):
    salt = 'xx01'
    keysalt = (masterkey[-4:] + salt) * 2
    try:
        hashed_mpass = finalhash(masterkey.encode(), keysalt.encode())
        return hashed_mpass
    except ValueError as e:
        print(e)



import pyperclip
import string
import random


def generate_password(passlen, lc=50, nc=30, pc=20):
    letters = list(string.ascii_letters)
    puncs = ['$', '#', '_', '-', '!', '@',
             '+', '&', '(', ')', '?', '.', '*', '%']
    digits = list(string.digits)

    def percent(char, pers):
        return int(pers*(char/100))

    password = ""
    password += "".join(random.choices(letters, k=percent(passlen, lc)))
    password += "".join(random.choices(puncs, k=percent(passlen, pc)))
    password += "".join(random.choices(digits, k=percent(passlen, nc)))

    breh = list(password)
    random.shuffle(breh)

    return "".join(breh)


def copy_to_clipboard(*argv, combo=False, divider=":"):
    """

    simply copy a string to clipboard or mixes gmail,password separated by `devider` 
    to make a combo and copy the full text into clipboard

    `*argv` : List of arguments

    `combo` ( options are either False or True ):
        False : Copies the first argument only .
        True : Copies the combo of all `argv` separated by `devider`

    `devider` : devides different argument

    e.g  : copy_to_clipboard( "This_is_A_pass" )

           copy_to_clipboard(

                        "helloworld@gmail.com",
                        "This_is_A_pass", 
                        combo=True , 
                        divider="-" 

                        )
    """

    if combo is False:
        pyperclip.copy(argv[0])
    else:
        combined_text = ""
        for arg in argv:
            if arg != argv[-1]:
                combined_text += str(arg)+divider
            else:
                combined_text += str(arg)

        pyperclip.copy(combined_text)


def show_hints(text, text_type="password", security=3, jokes=False):
    """
    text_types are :
        password
        gmail

    `security` : number of last digits to show as hint
    `jokes` : special feature for hint option . shows funny hints .

    """
    if jokes is False:
        amount_of_chars = len(text) - security + 1
        hidden_chars = "*" * amount_of_chars
        hint = text[0]+hidden_chars+text[-(security):]

        return (hint)

    else:
        pass
        # developments going on ( shoaib islam )



import argparse


def run_parser():

    parser = argparse.ArgumentParser(
        prog="ins",
        formatter_class=argparse.RawDescriptionHelpFormatter,
        description="",
        epilog="Read Docs at https://github.com/jis4nx/square-pass",
        conflict_handler="resolve",
        usage="%(prog)s action [show|remove|search] ",
    )

    flags = parser.add_argument_group("Command :", "")
    opt = parser.add_argument_group("Options :", "")

    opt.add_argument(
        "-U",
        "--update",
        dest="update",
        metavar="",
        help="Update your credential with service name",
    )
    opt.add_argument(
        "-l", "--login", dest="login", metavar="", nargs="?", const="None", help="Login to remember password"
    )
    opt.add_argument("-cat", "--cat", dest="cat", metavar="", help="View File")
    opt.add_argument(
        "-c", "--count", dest="count", metavar="", nargs=2, help="Counts reused credential"
    )
    flags.add_argument(
        "--ls",
        "--showlist",
        dest="showlist",
        nargs="?",
        const="list",
        help="Shows Credential",
    )
    flags.add_argument(
        "--rm", "--remove", dest="remove", metavar="", help="remove a credential"
    )

    flags.add_argument(
        "-n", "--normal", dest="normal", action="store_true", help="Show key while typing"
    )

# Insert
    flags.add_argument(
        "-P", "--passw", dest="passw", action="store_true", help="Add new credential"
    )
    opt.add_argument(
        "-K", "--keypass", dest="keypass", nargs="?", const="None", help="Add Key"
    )
    opt.add_argument(
        "-N", "--note", dest="note", metavar="", nargs="?", const="None", help="Add Note"
    )

# Filter
    opt.add_argument(
        "-u", "--username", dest="username", metavar="", help="Filter by Username"
    )
    opt.add_argument(
        "-a", "--appname", dest="appname", metavar="", help="Filter by Appname"
    )
    opt.add_argument(
        "-b",
        "--bp",
        dest="user_pass",
        action="store_true",
        help="Filter by both username and password",
    )
    opt.add_argument(
        "-i",
        "--ignorecase",
        dest="ignorecase",
        action="store_true",
        help="Index for the credential update",
    )

# Opt args
    opt.add_argument(
        "-C", "--copy", dest="copy", action="store_true", help="Copy to clipboard"
    )
    opt.add_argument(
        "-r",
        "--recent",
        dest="recent",
        action="store_true",
        help="Show recently modified credentials",
    )
# opt.add_argument("-W",'--warn',         action="store_true",                help="warn about weak passwords")

# Extra Args

    dan = parser.add_argument_group("Often Args :", "")
    dan.add_argument(
        "--bigbang",
        dest="bigbang",
        metavar="[boom | passw | keys | notes]",
        help="Erase Service information",
    )
    opt.add_argument(
        "-g",
        "--gen",
        dest="generate",
        nargs="?",
        type=int,
        const=8,
        help="Generate Advance & Strong Pass",
    )
    opt.add_argument(
        "-e", "--export", dest="export", nargs="+", help="Generate Advance & Strong Pass"
    )

    args = parser.parse_args()
    return args



from setuptools import setup, find_packages
import pathlib

here = pathlib.Path(__file__).parent.resolve()

long_description = open('README.md').read()


setup(
    name='square-pass3',
    version='0.0.6',
    description='CLI Based Password Manager',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/jis4nx/square-pass',
    author='Rakibul Islam Jisan',
    author_email='jis4nx@gmail.com',


    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3 :: Only',
    ],

    keywords='password_manager, encryption, aes256, cli-password-manager',
    packages=find_packages(),
    python_requires='>=3.6, <3.12',
    install_requires=[
        "pycryptodome==3.10.1",
        "prettytable==2.4.0",
        "colorama==0.4.4",
        "rich==10.15.2",
        "pyperclip",
        "psutil",
        "setuptools",
        "wheel",
        "PyYAML"
    ],

    entry_points={
        'console_scripts': [
            'sq-init=sqpass.install:setmain',
            'sq=sqpass.main:main',
        ],
    },
    project_urls={
        'Bug Reports': 'https://github.com/jis4nx/square-pass/issues',
        'Funding': 'https://donate.pypi.org',
        'Source': 'https://github.com/jis4nx/square-pass/',
    },
)



import os
from getpass import getpass
from sqpass.passwordManager import base
from sqpass.passwordManager import argaction
from sqpass.passwordManager.ciphers import hashuser, encrypt, decrypt
from sqpass.passwordManager.cache import set_with_ttl, CACHE_DIR
from sqpass.passwordManager.tools import is_process_running
from sqpass.passwordManager.parser import run_parser
from sqpass.passwordManager.base import readpass, pathname


from os import urandom

from subprocess import Popen

import pickle

pwdwrong = [
    "Sorry that's not correct!",
    "Not even close!",
    "Nice Try!",
    "Hold on and give the correct password!",
]

services = ["passw", "notes", "keys"]


class UserArgManager:
    def __init__(self, args):
        self.userInp = None
        self.db = None
        self.args = args

    def setup(self):
        data = None
        if os.path.exists(CACHE_DIR):
            with open(CACHE_DIR, 'rb') as file:
                data = pickle.load(file)

        if not os.path.exists(pathname):
            print('No Passkey found\nRun `sq-init` initialize')
            exit()
        if data:
            salt, iv, enc, rand_byte = data['upass'][0]
            upass = decrypt(salt, iv, enc, rand_byte).decode()
            self.userInp = upass
            self.db = base.DatabaseManager(
                self.userInp, hashuser(self.userInp))

            self.service_dict = {'passw': self.db.view_userpasses,
                                 'notes': self.db.view_notes,
                                 'keys': self.db.view_keys
                                 }
        else:
            userpass = getpass(prompt="Enter Masterpass: ")
            hashed_pass = hashuser(userpass)
            while True:
                if hashed_pass != readpass():
                    userpass = getpass(prompt="Try again: ")
                    hashed_pass = hashuser(userpass)
                else:
                    self.userInp = userpass
                    break
            self.db = base.DatabaseManager(
                self.userInp, hashuser(self.userInp))
            self.service_dict = {'passw': self.db.view_userpasses,
                                 'notes': self.db.view_notes,
                                 'keys': self.db.view_keys
                                 }

    def show_usage(self, show_services=True, usage_msg=None, example=None, show_fields=False, arg=None):
        print("")
        if show_services:
            print(f"Available Services -[ {' | '.join(services)} ]")
        if usage_msg is not None:
            print(f"usage: sq {arg} {usage_msg}")
        if example is not None:
            print(f"eg: sq {arg} {example}")
        if show_fields:
            pass
        print("")

    def handle_insert(self, passw=False, keypass=False, note=False):
        if passw:
            self.db.insert()
        if keypass:
            mode = False if self.args.normal else True
            self.db.keyins(title=self.args.keypass, silent=mode)
        if note:
            self.db.noteins(title=self.args.note)

    def generate_password(self):
        passw = argaction.generate_password(self.args.generate)
        if self.args.copy:
            argaction.copy_to_clipboard(passw)
        print(passw)

    def handle_retrieve_data(self):
        try:
            idx = int(self.args.cat.split("/")[1])
            service = self.args.cat.split("/")[0]
            try:
                self.service_dict[service](indexId=idx)
            except KeyError:
                print("Availables: ", " | ".join(services))
        except (IndexError, ValueError):
            print("Usage: sq -cat service/index")

    def handle_count(self):
        try:
            mode = True if self.args.ignorecase else False
            service = self.args.count[0].split("/")[0]
            field = self.args.count[0].split("/")[1]
            self.db.count(icase=mode, table=service,
                          column=field, cred=self.args.count[1])
        except Exception as err:
            print(err)

    def handle_showlist(self):
        sort = "date" if self.args.recent else "id"
        order = "DESC" if self.args.recent else "ASC"
        mode = True if self.args.ignorecase else False
        if self.args.username:
            self.db.filter(icase=mode, username=self.args.username)
        elif self.args.appname:
            self.db.filter(icase=mode, appname=self.args.appname)
        else:
            try:
                self.service_dict[self.args.showlist](sort=sort, order=order)
            except KeyError:
                self.show_usage(arg="--ls", example="passw",
                                usage_msg="<service>")

    def handle_remove(self):
        try:
            service_name = self.args.remove.split("/")[0]
            ind = int(self.args.remove.split("/")[1])
            try:
                self.db.remove_cd(table=service_name, u_id=ind)
            except KeyError:
                print("Available args:", " | ".join(services))
        except IndexError:
            print("Usage: sq --rm service/index")

    def handle_bigbang(self):
        if self.args.bigbang == "boom":
            self.db.bigbang(boom=True)
        if self.args.bigbang in services:
            self.db.bigbang(table_to_delete=self.args.bigbang)
        else:
            print("Available args: \nboom | passw | keys | notes")

    def handle_update(self):
        usage = r"sq -U <service>/<index>"
        eg = "sq -U passw/2"
        try:
            service_name = self.args.update.split("/")[0]
            index = int(self.args.update.split("/")[1])
            if service_name not in services:
                self.show_usage(show_services=True,
                                usage_msg=usage, example=eg)
                return
            try:
                self.db.update(table=service_name, id=index)
            except Exception as e:
                print(e)
                print("Availables: \nusers | keys")

        except Exception as e:
            self.show_usage(show_services=True,
                            usage_msg=usage, example=eg)

    def handle_export(self):
        try:
            service_name = self.args.export[0]
            fileform = self.args.export[1]
        except IndexError:
            fileform = "json"
        if fileform == "csv":
            self.db.export(service=service_name, csv=True)
        elif fileform == "json":
            self.db.export(service=service_name, json=True)

    def handle_login(self):
        if self.args.login:
            cache_time = input("Time duration to remember <minutes> : ")
            while True:
                if not cache_time.isdigit():
                    cache_time = input("Please Enter minutes in number: ")
                else:
                    break
            rand_byte = urandom(16)
            salt, iv, enc = encrypt(self.userInp.encode('utf-8'), rand_byte)
            set_with_ttl('upass', [salt, iv, enc, rand_byte], int(cache_time))
            path = os.path.join(os.path.dirname(
                os.path.abspath(__file__)), 'observer.py')
            if not is_process_running(path):
                Popen(['python', path], start_new_session=True)

    def initial_setup(self):
        pass

    def handle_functions(self):
        actions = {
            "update": self.handle_update,
            "export": self.handle_export,
            "bigbang": self.handle_bigbang,
            "remove": self.handle_remove,
            "showlist": self.handle_showlist,
            "cat": self.handle_retrieve_data,
            "note": lambda: self.handle_insert(note=self.args.note),
            "passw": lambda: self.handle_insert(passw=True),
            "keypass": lambda: self.handle_insert(keypass=True),
            "generate": self.generate_password,
            "login": self.handle_login,
        }

        for arg, action in actions.items():
            if getattr(self.args, arg):
                action()


def main():
    argmanager = UserArgManager(args=run_parser())
    argmanager.setup()
    argmanager.handle_functions()


if __name__ == "__main__":
    main()



from sqpass.passwordManager.ciphers import encrypt, decrypt
from sqpass.passwordManager.conf import get_config
from sqpass.create_db import check_db_file, create_database
import json
import psutil
from rich.table import Table
from rich import box
from rich.panel import Panel
from rich.markdown import Markdown
from rich.console import Console
from rich.layout import Layout
import sqlite3
from contextlib import contextmanager


def print_note(note, title, sub, markdown=True):

    console = Console()
    layout = Layout()

    note = Markdown(note) if markdown else note

    styles = Panel(
        note,
        box=box.ROUNDED,
        padding=(1, 2),
        title=f"[b red]{title}",
        border_style="bright_blue",
        subtitle=sub
    )

    console.print(styles)


def print_box(lst, master_pass):
    config = get_config()
    colors = config.get('passw_colors')
    table = Table()
    table.add_column("Index", style=colors.get('index'))
    table.add_column("Username", style=colors.get('username'))
    table.add_column("App Name", style=colors.get('appname'))
    table.add_column("Password", style=colors.get('password'))

    for row in lst:
        idx, username, app_name = row[:3]
        res = json.loads(row[3])
        salt = res['salt']
        iv = res['iv']
        cipher = res['enc']
        decipher = decrypt(
            salt, iv, cipher, master_pass.encode()).decode('utf-8')
        table.add_row(str(idx), username, app_name, decipher)
    console = Console()
    console.print(table)
    return ""


def is_process_running(command):
    for proc in psutil.process_iter(attrs=['cmdline']):
        try:
            if command in proc.info['cmdline']:
                return True
        except (psutil.NoSuchProcess, psutil.AccessDenied, psutil.ZombieProcess):
            return False
    return False


@contextmanager
def opendb(path):
    conn = sqlite3.connect(path)
    cur = conn.cursor()
    try:
        yield cur
    except (sqlite3.DatabaseError, sqlite3.OperationalError) as e:
        raise
    finally:
        conn.commit()
        conn.close()


@contextmanager
def dbfetch(path, query, data=None):
    try:
        conn = sqlite3.connect(path)
        cur = conn.cursor()
        if data is None:
            cur.execute(query)
        else:
            cur.execute(query, data)
        yield cur.fetchall()
    except (sqlite3.DatabaseError) as e:
        print("Database not found, Try to run `sq-init`")
        exit()
    finally:
        cur.close()
